<?php

namespace app\common\model\I18n\I18nLang;

use app\common\arw\adjfut\src\Validate;
use app\BaseModel;
use think\model\concern\SoftDelete;
use app\common\arw\adjfut\src\Excel;
use app\Request;
use app\common\exception\Tool;
use think\facade\Db;
use app\common\model\I18n\I18nLangType\I18nLangType as ModelI18nLangType;

class I18nLang extends BaseModel
{
    use SoftDelete;
    // 删除字段
    protected $deleteTime = 'i18n_lang_delete_time';
    // 设置主键名
    protected $pk = 'i18n_lang_guid';
    // 设置废弃字段
    protected $disuse = [];
    // 设置字段信息
    protected $schema = [
        'i18n_lang_id' => 'int',
        'i18n_lang_guid' => 'string',
        'i18n_lang_type_guid' => 'string',
        'i18n_lang_key' => 'string',
        'i18n_lang_name' => 'string',
        'i18n_lang_create_time' => 'datetime',
        'i18n_lang_create_user_guid' => 'string',
        'i18n_lang_update_time' => 'datetime',
        'i18n_lang_update_user_guid' => 'string',
        'i18n_lang_delete_time' => 'datetime',
        'i18n_lang_delete_user_guid' => 'string',

    ];
    // 设置json类型字段
    protected $json = [''];
    // 开启自动写入时间戳字段
    protected $autoWriteTimestamp = 'datetime';
    // 创建时间
    protected $createTime = 'i18n_lang_create_time';
    // 修改时间
    protected $updateTime = 'i18n_lang_update_time';

    // excel导入/下载模板表头
    public const EXCELFIELD = [
        'i18n_lang_type_code' => '语言类型code码',
        'i18n_lang_key' => '语言key',
        'i18n_lang_name' => '语言name',
    ];

    /**
     * 新增前
     */
    public static function onBeforeInsert(self $model): void
    {
        // self::checkRepeatData($model);
        $model->completeCreateField();
    }

    /**
     * 更新前
     */
    public static function onBeforeUpdate(self $model): void
    {
        // self::checkRepeatData($model);
        $model->completeUpdateField();
    }

    /**
     * 删除前
     */
    public static function onBeforeDelete(self $model): void
    {
        $model->completeDeleteField();
    }


    /**
     * 导出Excel
     * 
     * @param array $select 导出的数据
     */
    public static function exportExcel(array $select): void
    {
        $data = [[
            '语言类型code码',
            '语言key',
            '语言name'
        ]];
        foreach ($select as $key => $val) {
            // 初始化语言类型转换
            $i18n_lang_type = ModelI18nLangType::where('i18n_lang_type_guid',$val['i18n_lang_type_guid'])->find();
            if (!$i18n_lang_type) throwErrorMsg("语言类型code码:【{$val['i18n_lang_type_code']}】不存在");
            $i18n_lang_type_code = $i18n_lang_type->i18n_lang_type_code;

            $data[] = [
                $i18n_lang_type_code,
                $val['i18n_lang_key'],
                $val['i18n_lang_name'],
            ];
        }
        $excel = (new Excel())->exporTsheet($data);
        $excel->save('多语言设置.xlsx');
    }


    /**
     * 导入excel
     * 
     * @param \app\common\arw\adjfut\src\UploadFile $file excel
     */
    public static function importExcel(\app\common\arw\adjfut\src\UploadFile $file): string
    {
        $msg = [];

        Db::startTrans();
        try {
            $excel = new Excel($file);
            $data = $excel->parseExcel(
                Tool::getExcelRule(self::EXCELFIELD),
                ['titleLine' => [1]]
            );
            if (!$data)  throwErrorMsg('excel无数据', 1);
            $msg = [];
            foreach ($data as $line => $value) {
                try {
                    $model = self::importExcelInit($value);
                    $msg[] = "{$line} <span style='color:#27af49'>多语言设置:【{$value['i18n_lang_type_code']}: {$value['i18n_lang_key']}】新增成功!</span><br>";
                } catch (\Throwable $th) {
                    $msg[] = "{$line} <span style='color:red'>{$th->getMessage()}</span><br>";
                }
            }
            Db::commit();
            return implode(', ', $msg);
        } catch (\Throwable $th) {
            Db::rollback();
            throw $th;
        }
    }


    /**
     * 导入excel初始化
     * 
     * @param array $value excel每行数据
     */
    public static function importExcelInit(array $value): void
    {
        $i18n_lang_type_code = $value['i18n_lang_type_code'];
        $i18n_lang_key = $value['i18n_lang_key'];
        $i18n_lang_name = $value['i18n_lang_name'];

        // 初始化语言类型转换
        $i18n_lang_type = ModelI18nLangType::where('i18n_lang_type_code', $i18n_lang_type_code)->find();
        if (!$i18n_lang_type) throwErrorMsg("语言类型code码:【{$i18n_lang_type_code}】不存在");
        $i18n_lang_type_guid = $i18n_lang_type->i18n_lang_type_guid;

        $find = self::where("i18n_lang_type_guid",$i18n_lang_type_guid)->find()->where("i18n_lang_key",$i18n_lang_key)->where("i18n_lang_name",$i18n_lang_name)->find();
        if ($find) throwErrorMsg("语言类型code码:【{$i18n_lang_type_code}】- 语言key:【{$i18n_lang_key}】- 语言name:【{$i18n_lang_name}】 已存在，请检查");
        // if($find) return;

        self::create(
            [
                'i18n_lang_type_guid' => $i18n_lang_type_guid,
                'i18n_lang_key' => $i18n_lang_key,
                'i18n_lang_name' => $i18n_lang_name,
            ],
            [
                'i18n_lang_type_guid',
                'i18n_lang_key',
                'i18n_lang_name',
                'i18n_lang_guid',
                'i18n_lang_create_user_guid',
                'i18n_lang_update_user_guid'
            ]
        );
    }
}
