<?php

namespace app\common\model\Products;

use app\common\arw\adjfut\src\Validate;
use app\BaseModel;
use think\model\concern\SoftDelete;
use app\common\arw\adjfut\src\Excel;
use app\Request;
use app\common\exception\Tool;
use think\facade\Db;
use app\common\model\Products\ProductType as ModelProductType;
use app\common\model\I18n\I18nLangType\I18nLangType as ModelI18nLangType;


class ProductTypeLang extends BaseModel
{
    use SoftDelete;
    // 删除字段
    protected $deleteTime = 'product_type_lang_delete_time';
    // 设置主键名
    protected $pk = 'product_type_lang_guid';
    // 设置废弃字段
    protected $disuse = [];
    // 设置字段信息
    protected $schema = [
        'product_type_lang_id' => 'int',
        'product_type_lang_guid' => 'string',
        'product_type_guid' => 'string',
        'i18n_lang_type_guid' => 'string',
        'product_type_lang_name' => 'string',
        'product_type_lang_create_time' => 'datetime',
        'product_type_lang_create_user_guid' => 'string',
        'product_type_lang_update_time' => 'datetime',
        'product_type_lang_update_user_guid' => 'string',
        'product_type_lang_delete_time' => 'datetime',
        'product_type_lang_delete_user_guid' => 'string',

    ];
    // 设置json类型字段
    protected $json = [''];
    // 开启自动写入时间戳字段
    protected $autoWriteTimestamp = 'datetime';
    // 创建时间
    protected $createTime = 'product_type_lang_create_time';
    // 修改时间
    protected $updateTime = 'product_type_lang_update_time';

    // excel导入/下载模板表头
    public const EXCELFIELD = [
        'i18n_lang_type_guid' => '语言类型code',
        'product_type_guid' => '所属产品类型',
        'product_type_lang_name' => '产品类型多语言名称',
    ];

    /**
     * 新增前
     */
    public static function onBeforeInsert(self $model): void
    {
        // self::checkRepeatData($model);
        $model->completeCreateField();
    }

    /**
     * 更新前
     */
    public static function onBeforeUpdate(self $model): void
    {
        // self::checkRepeatData($model);
        $model->completeUpdateField();
    }

    /**
     * 删除前
     */
    public static function onBeforeDelete(self $model): void
    {
        $model->completeDeleteField();
    }


    /**
     * 导出Excel
     * 
     * @param array $select 导出的数据
     */
    public static function exportExcel(array $select): void
    {
        $data = [[
            '语言类型code',
            '所属产品类型',
            '产品类型多语言名称'
        ]];
        foreach ($select as $key => $val) {
            $data[] = [
                $val['i18n_lang_type_code'],
                $val['product_type_name'],
                $val['product_type_lang_name'],
            ];
        }
        $excel = (new Excel())->exporTsheet($data);
        $excel->save('产品类型多语言.xlsx');
    }


    /**
     * 导入excel
     * 
     * @param \app\common\arw\adjfut\src\UploadFile $file excel
     */
    public static function importExcel(\app\common\arw\adjfut\src\UploadFile $file): string
    {
        $msg = [];

        Db::startTrans();
        try {
            $excel = new Excel($file);
            $data = $excel->parseExcel(
                Tool::getExcelRule(self::EXCELFIELD),
                ['titleLine' => [1]]
            );
            if (!$data)  throwErrorMsg('excel无数据', 1);
            $msg = [];
            foreach ($data as $line => $value) {
                try {
                    $model = self::importExcelInit($value);
                    $msg[] = "{$line} <span style='color:#27af49'>【{$value['product_type_lang_name']}】新增成功!</span><br>";
                } catch (\Throwable $th) {
                    $msg[] = "{$line} <span style='color:red'>{$th->getMessage()}</span><br>";
                }
            }
            Db::commit();
            return implode(', ', $msg);
        } catch (\Throwable $th) {
            Db::rollback();
            throw $th;
        }
    }


    /**
     * 导入excel初始化
     * 
     * @param array $value excel每行数据
     */
    public static function importExcelInit(array $value): void
    {
        $product_type_guid = $value['product_type_guid'];
        $i18n_lang_type_code = $value['i18n_lang_type_guid'];
        $product_type_lang_name = $value['product_type_lang_name'];

        $i18n_lang_type = ModelI18nLangType::where('i18n_lang_type_code', $i18n_lang_type_code)->find();
        if (!$i18n_lang_type) throwErrorMsg("语言类型code码:【{$i18n_lang_type_code}】不存在");
        $i18n_lang_type_guid = $i18n_lang_type->i18n_lang_type_guid;

        $product_type = ModelProductType::where('product_type_name', $product_type_guid)->find();
        if (!$product_type) throwErrorMsg("产品类目:【{$product_type_guid}】不存在");
        $product_type_guid = $product_type->product_type_guid;

        $same_data = self::where('product_type_lang_name', $product_type_lang_name)
            ->where('i18n_lang_type_guid', $i18n_lang_type_guid)->find();
        if ($same_data) throwErrorMsg("多语言产品类目:【{$product_type_lang_name}】相同语种已有相同数据");

        self::create(
            [
                'product_type_guid' => $product_type_guid,
                'i18n_lang_type_guid' => $i18n_lang_type_guid,
                'product_type_lang_name' => $product_type_lang_name,
            ],
            [
                'product_type_guid',
                'i18n_lang_type_guid',
                'product_type_lang_name',
                'product_type_lang_guid',
                'product_type_lang_create_user_guid',
                'product_type_lang_update_user_guid'
            ]
        );
    }
}
